@@ -4,8 +4,8 @@
 ### reflection tools (inofficial so far, because it returns strings
 ### instead of the reflection objects - it's useful for quick checks,
 ### though)
-.jmethods <- function(o, name=NULL, as.obj=FALSE) {
-  cl <- if (is(o, "jobjRef")) .jcall(o, "Ljava/lang/Class;", "getClass") else if (is(o, "jclassName")) o@jobj else .jfindClass(as.character(o))
+.jmethods <- function(o, name=NULL, as.obj=FALSE, class.loader=.rJava.class.loader) {
+  cl <- if (is(o, "jobjRef")) .jcall(o, "Ljava/lang/Class;", "getClass") else if (is(o, "jclassName")) o@jobj else .jfindClass(as.character(o), class.loader=class.loader)
   ms<-.jcall(cl,"[Ljava/lang/reflect/Method;","getMethods")
   if (length(name)) {
      n <- sapply(ms, function(o) .jcall(o, "S", "getName"))
@@ -14,23 +14,27 @@
   if (isTRUE(as.obj)) ms else unlist(lapply(ms,function(x) .jcall(x,"S","toString")))
 }
 
-.jconstructors <- function(o, as.obj=FALSE) {
-  cl <- if (is(o, "jobjRef")) .jcall(o, "Ljava/lang/Class;", "getClass") else if (is(o, "jclassName")) o@jobj else .jfindClass(as.character(o))
+.jconstructors <- function(o, as.obj=FALSE, class.loader=.rJava.class.loader) {
+  cl <- if (is(o, "jobjRef")) .jcall(o, "Ljava/lang/Class;", "getClass") else if (is(o, "jclassName")) o@jobj else .jfindClass(as.character(o), class.loader=class.loader)
   cs<-.jcall(cl,"[Ljava/lang/reflect/Constructor;","getConstructors")
   if (isTRUE(as.obj)) return(cs)
   unlist(lapply(cs,function(x) .jcall(x,"S","toString")))
 }
 
 ### this list maps R class names to Java class names for which the constructor does the necessary conversion (for use in .jrcall)
-.class.to.jclass <-    c(character= "java/lang/String",
-                         jbyte    = "java/lang/Byte",
-                         integer  = "java/lang/Integer",
-                         numeric  = "java/lang/Double",
-                         logical  = "java/lang/Boolean",
-                         jlong    = "java/lang/Long",
-                         jchar    = "java/lang/Character",
-                         jshort   = "java/lang/Short",
-                         jfloat   = "java/lang/Float")
+### The order matters in case an object inherits from multiple (S3) classes - then the first one
+### in the list is picked.
+.class.to.jclass <-    c(
+    jlong    = "java/lang/Long",
+    jchar    = "java/lang/Character",
+    jshort   = "java/lang/Short",
+    jfloat   = "java/lang/Float",
+    jbyte    = "java/lang/Byte",
+    integer  = "java/lang/Integer",
+    numeric  = "java/lang/Double",
+    logical  = "java/lang/Boolean",
+    character= "java/lang/String"
+)
 
 ### Java classes that have a corresponding primitive type and thus a corresponding TYPE field to use with scalars
 .primitive.classes = c("java/lang/Byte", "java/lang/Integer", "java/lang/Double", "java/lang/Boolean",
@@ -42,17 +46,23 @@
 # this is used for internal purposes only, in particular 
 # it does not dispatch arrays to jrectRef
 ._java_valid_object <- function(a) {
-  if (is(a, "jobjRef")) a 
-  else if (is.null(a)) .jnull() else {
-    cm <- match(class(a)[1], names(.class.to.jclass))
-    if (!any(is.na(cm))) { 
-    	if (length(a) == 1) { 
+  if (is(a, "jobjRef")) a
+  else if (is(a, "jclassName")) a@jobj
+  else if (is.null(a)) .jnull()
+  else if (is.raw(a)) .jarray(a, dispatch=FALSE) ## raw is always [B
+  else {
+      ## check all classes (in S3 case, see #317)
+      cm <- match(class(a), names(.class.to.jclass))
+      if (!all(is.na(cm))) { ## at least one subclass matches
+          cm <- min(cm, na.rm=TRUE) ## pick the lowest (i.e. in precedence of the order above)
+          ## scalar? then create directly
+          if (length(a) == 1) {
     		y <- .jnew(.class.to.jclass[cm], a)
     		if (.class.to.jclass[cm] %in% .primitive.classes) attr(y, "primitive") <- TRUE
-    		y 
+                y
     	} else .jarray(a, dispatch = FALSE)
     } else {
-      stop("Sorry, parameter type `", cm ,"' is ambiguous or not supported.")
+      stop("Sorry, parameter type `", class(a)[1] ,"' is ambiguous or not supported.")
     }
   }
 }
@@ -68,6 +78,7 @@
   isTRUE(attr(x, "primitive"))
 
 ._java_class <- function( x ){
+  ## FIXME: not sure how to pass though the class loader here - only affects NULL objects
   if (is.jnull(x)) { if (is(x,"jobjRef")) .jfindClass(x@jclass) else .jclassObject } else {
     if (._isPrimitiveReference(x)) .jfield(x, "Ljava/lang/Class;", "TYPE") else .jcall(x, "Ljava/lang/Class;", "getClass")
   }
@@ -78,14 +89,17 @@
 ### reflected call - this high-level call uses reflection to call a method
 ### it is much less efficient than .jcall but doesn't require return type
 ### specification or exact matching of parameter types
-.jrcall <- function(o, method, ..., simplify=TRUE) {
+.jrcall <- function(o, method, ..., simplify=TRUE, class.loader=.rJava.class.loader) {
   if (!is.character(method) | length(method) != 1)
     stop("Invalid method name - must be exactly one character string.")
-  if (inherits(o, "jobjRef") || inherits(o, "jarrayRef"))
+  if (.need.init()) .jinit()
+  if (is(o, "jclassName"))
+    cl <- o@jobj
+  else if (is(o, "jobjRef"))
     cl <- .jcall(o, "Ljava/lang/Class;", "getClass")
   else
-    cl <- .jfindClass(o)
-  if (is.null(cl))
+    cl <- .jfindClass(o, class.loader=class.loader)
+  if (is.jnull(cl))
     stop("Cannot find class of the object.")
   
   # p is a list of parameters that are formed solely by valid Java objects
@@ -125,10 +139,16 @@
 ### on the classes of the ... it does not require exact match between 
 ### the objects and the constructor parameters
 ### This is to .jnew what .jrcall is to .jcall
-.J <- function(class, ...) {
-  # allow non-JNI specifiation
-  class <- gsub("\\.","/",class) 
-  
+.J <- function(class, ..., class.loader=.rJava.class.loader) {
+    ## if it is a name, load the class, otherwise assume we have the object
+    if (is.character(class)) {
+        ## allow non-JNI specifiation
+        class <- gsub("\\.","/",class)
+        class <- .jfindClass(class, class.loader=class.loader)
+    } else if (is(class, "jclassName")) {
+        class <- class@jobj
+    }
+
   # p is a list of parameters that are formed solely by valid Java objects
   p <- ._java_valid_objects_list(...)
   
@@ -137,7 +157,7 @@
 
   # use RJavaTools to find create the object
   o <- .jcall("RJavaTools", "Ljava/lang/Object;", 
-  	"newInstance", .jfindClass(class), 
+        "newInstance", class,
   	.jarray(p,"java/lang/Object", dispatch = FALSE ), 
   	.jarray(pc,"java/lang/Class", dispatch = FALSE ), 
   	evalString = FALSE, evalArray = FALSE, use.true.class = TRUE )
@@ -187,8 +207,8 @@
 #! }
 
 ### list the fields of a class or object
-.jfields <- function(o, name=NULL, as.obj=FALSE) {
-  cl <- if (is(o, "jobjRef")) .jcall(o, "Ljava/lang/Class;", "getClass") else if (is(o, "jclassName")) o@jobj else .jfindClass(as.character(o))
+.jfields <- function(o, name=NULL, as.obj=FALSE, class.loader=.rJava.class.loader) {
+  cl <- if (is(o, "jobjRef")) .jcall(o, "Ljava/lang/Class;", "getClass") else if (is(o, "jclassName")) o@jobj else .jfindClass(as.character(o), class.loader=class.loader)
   f <- .jcall(cl, "[Ljava/lang/reflect/Field;", "getFields")
   if (length(name)) {
      n <- sapply(f, function(o) .jcall(o, "S", "getName"))
@@ -220,20 +240,20 @@ hasClass <- function( x, name){
 }
 
 ### the following ones are needed for the static version of $
-classHasField <- function(x, name, static=FALSE) {
-	if (is(x, "jclassName")) x <- x@jobj else if (!is(x, "jobjRef")) x <- .jfindClass(as.character(x))
+classHasField <- function(x, name, static=FALSE, class.loader=.rJava.class.loader) {
+	if (is(x, "jclassName")) x <- x@jobj else if (!is(x, "jobjRef")) x <- .jfindClass(as.character(x), class.loader=class.loader)
 	._must_be_character_of_length_one(name)
 	.jcall("RJavaTools", "Z", "classHasField", x, name, static)
 }
 
-classHasMethod <- function(x, name, static=FALSE) {
-	if (is(x, "jclassName")) x <- x@jobj else if (!is(x, "jobjRef")) x <- .jfindClass(as.character(x))
+classHasMethod <- function(x, name, static=FALSE, class.loader=.rJava.class.loader) {
+	if (is(x, "jclassName")) x <- x@jobj else if (!is(x, "jobjRef")) x <- .jfindClass(as.character(x), class.loader=class.loader)
 	._must_be_character_of_length_one(name)
 	.jcall("RJavaTools", "Z", "classHasMethod", x, name, static)
 }
 
-classHasClass <- function(x, name, static=FALSE) {
-	if (is(x, "jclassName")) x <- x@jobj else if (!is(x, "jobjRef")) x <- .jfindClass(as.character(x))
+classHasClass <- function(x, name, static=FALSE, class.loader=.rJava.class.loader) {
+	if (is(x, "jclassName")) x <- x@jobj else if (!is(x, "jobjRef")) x <- .jfindClass(as.character(x), class.loader=class.loader)
 	._must_be_character_of_length_one(name)
 	.jcall("RJavaTools", "Z", "classHasClass", x, name, static)
 }
