@@ -633,19 +633,25 @@ public class RJavaArrayTools {
 		
 		Method m = getCloneMethod( o.getClass() ) ;
 		boolean access = m.isAccessible() ; 
-		m.setAccessible( true ) ;
+		if (!access) {
+			try {
+				m.setAccessible( true ) ;
+			} catch (Throwable e) {
+				access = true; /* give up */
+			}
+		}
+
 		try{
 			for( int i=0; i<size; i++){
 				Object cloned = o.getClass().cast( m.invoke( o, (Object[])null ) );
 				res[i] = cloned ;
 			}
-		} catch( IllegalAccessException e) {
-			m.setAccessible( access );
-			/* should not happen */
 		} catch( InvocationTargetException e){
-			m.setAccessible( access );
 			throw e.getCause() ; 
-		} 
+		} finally {
+			if (!access)
+				m.setAccessible( access );
+		}
 		return res ;
 	}
 	
@@ -670,19 +676,25 @@ public class RJavaArrayTools {
 	public static Object cloneObject( Object o) throws Throwable {
 		Method m = getCloneMethod( o.getClass() ) ;
 		boolean access = m.isAccessible() ; 
-		m.setAccessible( true ) ;
+		if (!access) {
+			try {
+				m.setAccessible( true ) ;
+			} catch (Throwable e) {
+				access = true; /* give up */
+			}
+		}
 		
 		Object copy = null ; 
 		
-		try{
-				copy = o.getClass().cast( m.invoke( o, (Object[])null ) );
-		} catch( IllegalAccessException e) {
-			m.setAccessible( access );
-			/* should not happen */
+		try {
+			copy = o.getClass().cast( m.invoke( o, (Object[])null ) );
 		} catch( InvocationTargetException e){
-			m.setAccessible( access );
 			throw e.getCause() ; 
+		} finally {
+			if (!access)
+				m.setAccessible( access );
 		}
+
 		return copy ;
 	}
 	// }}}
@@ -726,7 +738,7 @@ public class RJavaArrayTools {
 		if (d == null) return null;
 		int i = 0, n = d.length;
 		Double o[] = new Double[i];
-		for (i = 0; i < n; i++) if (!isNA(d[i])) o[i] = new Double(d[i]);
+		for (i = 0; i < n; i++) if (!isNA(d[i])) o[i] = Double.valueOf(d[i]);
 		return o;
 	}
 
@@ -734,7 +746,7 @@ public class RJavaArrayTools {
 		if (d == null) return null;
 		int i = 0, n = d.length;
 		Integer o[] = new Integer[i];
-		for (i = 0; i < n; i++) if (d[i] != NA_INTEGER) o[i] = new Integer(d[i]);
+		for (i = 0; i < n; i++) if (d[i] != NA_INTEGER) o[i] = Integer.valueOf(d[i]);
 		return o;
 	}
 
@@ -742,7 +754,7 @@ public class RJavaArrayTools {
 		if (d == null) return null;
 		int i = 0, n = d.length;
 		Boolean o[] = new Boolean[i];
-		for (i = 0; i < n; i++) if (d[i] != NA_INTEGER) o[i] = new Boolean((d[i] == 0) ? false : true);
+		for (i = 0; i < n; i++) if (d[i] != NA_INTEGER) o[i] = Boolean.valueOf((d[i] == 0) ? false : true);
 		return o;
 	}
 }
