@@ -1,7 +1,7 @@
 package org.rosuda.REngine.Rserve;
 
 // JRclient library - client interface to Rserve, see http://www.rosuda.org/Rserve/
-// Copyright (C) 2004-08 Simon Urbanek
+// Copyright (C) 2004-21 Simon Urbanek
 // --- for licensing information see LICENSE file in the original JRclient distribution ---
 
 import java.util.*;
@@ -25,6 +25,11 @@ public class RConnection extends REngine {
     String Key=null;
     RTalk rt=null;
 
+    REXP capabilities = null;
+    OOBInterface oob = null;
+
+    boolean isOCAP = false;
+
     String host;
     int port;
 
@@ -112,6 +117,24 @@ public class RConnection extends REngine {
 	initWithSocket(sock, null);
     }
 
+    /** set OOB callbacks, i.e. an object that will handle OOB_SEND and OOB_MSG packets in OCAP mode
+	@param callbacks object implementing the OOB interface */
+    public void setOOB(OOBInterface callbacks) {
+	oob = callbacks;
+    }
+
+    /** initialization in OCAP mode, assumes all communication variables are setup already, expected to be called
+	by initWithSocket */
+    private void initOCAP(Socket sock, byte[] header) throws RserveException {
+	/* there is no version in OCAP but 103 is assumed since that is
+	   the earliest version that supports OCAPs */
+	rsrvVersion = 103;
+	isOCAP = true;
+	connected = true;
+	RPacket rp = rt.response(header);
+	capabilities = parseEvalResponse(rp);
+    }
+
     private void initWithSocket(Socket sock, RSession session) throws RserveException {
 	s = sock;
         try {
@@ -130,10 +153,28 @@ public class RConnection extends REngine {
 			    throw new RserveException(this, "Error while receiving data: "+sre.getMessage(), sre);
 			}
 			try {
+				/* is this OCAP mode ? */
+                                if (n >= 16 &&
+				    IDs[0] == 0x52 && IDs[1] == 0x73 &&
+                                    IDs[2] == 0x4f && IDs[3] == 0x43) {
+					/* it is possible that the buffering doesn't work out
+					   and the first packet is <32 bytes, in which case
+					   we have to re-wrap the array to have the correct length */
+                                        if (n < 32) {
+                                                byte[] header = new byte[n];
+                                                System.arraycopy(IDs, 0, header, 0, n);
+                                                IDs = header;
+                                        }
+
+                                        initOCAP(sock, IDs);
+                                        return;
+                                }
+
 				if (n!=32) {
 					throw new RserveException(this,"Handshake failed: expected 32 bytes header, got "+n);
 				}
-				String ids=new String(IDs);
+				String ids = new String(IDs);
+				/* regular Rserve mode? */
 				if (ids.substring(0,4).compareTo("Rsrv")!=0)
 					throw new RserveException(this, "Handshake failed: Rsrv signature expected, but received \""+ids+"\" instead.");
 				try {
@@ -197,7 +238,20 @@ public class RConnection extends REngine {
         } catch(Exception e) { };
 		return false;
     }
-    
+
+    /** Returns capabilities received from the server on connect.
+	For non-OCAP mode this always returns <code>null</code> */
+    public REXP capabilities() {
+	return capabilities;
+    }
+
+    /** Check whether this connection is to Rserve in OCAP mode. Note that <code>callOC</code> is
+	the only command allowed in OCAP mode.
+	@return <code>true</code> if this connection is in OCAP mode, <code>false</code> otherwise. */
+    public boolean isOCAP() {
+	return isOCAP;
+    }
+
     /** evaluates the given command, but does not fetch the result (useful for assignment
 	operations)
 	@param cmd command/expression string */
@@ -295,6 +349,41 @@ public class RConnection extends REngine {
 	}
     }
 
+    public REXP callOCAP(REXP call) throws RserveException {
+	if (!connected || rt == null)
+	    throw new RserveException(this, "Not connected");
+	if (!isOCAP)
+	    throw new RserveException(this, "callOCAP is only available in OCAP mode");
+
+	RPacket rp = rt.request(RTalk.CMD_OCcall, call);
+	/* process any OOB messages */
+	while (rp != null && rp.isOOB()) {
+	    REXP payload = parseEvalResponse(rp);
+	    if ((rp.getCmd() & 0xff000) == RTalk.OOB_SEND) {
+		if (oob != null)
+		    oob.oobSend(rp.getCmd() & 0xfff, payload);
+		rp = rt.response();
+	    } else if ((rp.getCmd() & 0xff000) == RTalk.OOB_MSG) {
+		if (oob == null)
+		    throw new RserveException(this, "OOB_MSG received, but no OOB listener registered", rp);
+		REXP res = oob.oobMessage(rp.getCmd() & 0xfff, payload);
+		if (res == null)
+		    throw new RserveException(this, "OOB_MSG callback returned null", rp);
+		    // FIXME: we don't have official documentation for this - what is the
+		    // correct response to OOB_MSG? rserve-js uses cmd | RESP_OK/RESP_ERR
+		    // but that mangles low two bits of the OOB code
+		    // Rserve itself doesn't care so it is technically undefined
+		    // The most senstible thing to do is to just copy the cmd since
+		    // the recieving end knows that this is a msg response
+		rp = rt.request(rp.getCmd(), res);
+	    } else
+		throw new RserveException(this, "Unsupported OOB command received", rp);
+	}
+	if (rp == null || !rp.isOk())
+	    throw new RserveException(this,"callOCAP failed", rp);
+	return parseEvalResponse(rp);
+    }
+
     /** assign a content of a REXP to a symbol in R. The symbol is created if it doesn't exist already.
      * @param sym symbol name. Currently assign uses CMD_setSEXP command of Rserve, i.e. the symbol value is NOT parsed. It is the responsibility of the user to make sure that the symbol name is valid in R (recall the difference between a symbol and an expression!). In fact R will always create the symbol, but it may not be accessible (examples: "bar\nfoo" or "bar$foo").
 	 * @param rexp contents
